import { Body, Controller, Get, Param, Post } from '@nestjs/common';
import { ApiTags } from '@nestjs/swagger';
import { Public } from 'src/core/guards';
import { TransactionDto } from './dto/booking-order.dto';
import { TransactionEntity } from 'src/modules/transaction/transaction/domain/entities/transaction.entity';
import { TransactionDataService } from 'src/modules/transaction/transaction/data/services/transaction-data.service';
import { TABLE_NAME } from 'src/core/strings/constants/table.constants';
import { MidtransService } from 'src/modules/configuration/midtrans/data/services/midtrans.service';
import { CreateBookingManager } from '../domain/usecases/managers/create-booking.manager';

@ApiTags('Booking Order')
@Controller('v1/booking')
@Public(true)
export class BookingOrderController {
  constructor(
    private createBooking: CreateBookingManager,
    private serviceData: TransactionDataService,
    private midtransService: MidtransService,
  ) {}

  @Post()
  async create(@Body() data: TransactionDto) {
    const payload: Partial<TransactionEntity> = data;

    this.createBooking.setData(payload as any);
    this.createBooking.setService(
      this.serviceData,
      TABLE_NAME.TRANSACTION,
      this.midtransService,
    );
    await this.createBooking.execute();
    const result = await this.createBooking.getResult();
    const {
      invoice_code,
      status,
      payment_midtrans_token,
      payment_midtrans_url,
      id,
    } = result;

    return {
      id,
      invoice_code,
      status,
      payment_midtrans_token,
      payment_midtrans_url,
    };
  }

  @Get(':id')
  async get(@Param('id') transactionId: string) {
    const data = await this.serviceData.getOneByOptions({
      where: { id: transactionId },
    });

    const {
      customer_name,
      customer_phone,
      booking_date,
      invoice_code,
      status,
      id,
    } = data;

    return {
      customer_name,
      customer_phone,
      booking_date,
      invoice_code,
      status,
      id,
    };
  }
}
